/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */
package org.dive4elements.river.artifacts.sinfo.flood_duration;

import org.apache.commons.lang.math.DoubleRange;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.WINFOArtifact;
import org.dive4elements.river.artifacts.model.Calculation;
import org.dive4elements.river.artifacts.model.CalculationResult;
import org.dive4elements.river.artifacts.resources.Resources;
import org.dive4elements.river.artifacts.sinfo.SINFOArtifact;
import org.dive4elements.river.artifacts.sinfo.common.RiverInfoProvider;
import org.dive4elements.river.artifacts.sinfo.flood_duration.RiversideRadioChoice.RiversideChoiceKey;
import org.dive4elements.river.artifacts.sinfo.tkhstate.WinfoArtifactWrapper;
import org.dive4elements.river.artifacts.sinfo.util.CalculationUtils;
import org.dive4elements.river.artifacts.sinfo.util.RiverInfo;
import org.dive4elements.river.model.River;

/**
 * Calculation of the flood durations of the infrastructures of the km range of a river
 *
 * @author Matthias Schäfer
 */
final class FloodDurationCalculation {

    private final CallContext context;

    public FloodDurationCalculation(final CallContext context) {
        this.context = context;
    }

    public CalculationResult calculate(final SINFOArtifact sinfo) {

        final String user = CalculationUtils.findArtifactUser(this.context, sinfo);

        /* access input data */
        final FloodDurationAccess access = new FloodDurationAccess(sinfo);
        final River river = access.getRiver();
        final RiverInfo riverInfo = new RiverInfo(river);
        final DoubleRange calcRange = access.getRange();

        final RiverInfoProvider infoProvider = RiverInfoProvider.forRange(this.context, river, calcRange);
        final String calcModeLabel = Resources.getMsg(this.context.getMeta(), sinfo.getCalculationMode().name());
        final String label = Resources.getMsg(this.context.getMeta(), access.getRiverside().getKey());

        final Calculation problems = new Calculation();

        // Calculate the selected main values, if any
        /* misuse winfo-artifact to calculate waterlevels in the same way */
        final WINFOArtifact winfo = new WinfoArtifactWrapper(sinfo);
        winfo.addStringData("ld_mode", "distance");
        winfo.addStringData("ld_step", "100");

        final FloodDurationCalculationResults results = new FloodDurationCalculationResults(calcModeLabel, user, riverInfo, calcRange);

        final FloodDurationCalculationResult result = calculateResult(label, calcRange, infoProvider, access.getRiverside(), problems, winfo);
        results.addResult(result, problems);

        return new CalculationResult(results, problems);
    }

    /**
     * Calculates the flood durations of the infrastructures of a km range of a river
     */
    private FloodDurationCalculationResult calculateResult(final String label, final DoubleRange calcRange, final RiverInfoProvider riverInfoProvider,
            final RiversideChoiceKey riverside, final Calculation problems, final WINFOArtifact winfo) {

        final FloodDurationCalculator calculator = new FloodDurationCalculator(this.context, riverInfoProvider);
        return calculator.execute(problems, label, calcRange, riverside, winfo);
    }
}